
#include <stdio.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/comms/main.h"
#include "impulse/comms/channel.h"
#include "impulse/util/memory.h"
//#include "impulse_hash.h"


//impulse_hash clients;

void client_die(impulse_channel client)
{
  impulse_unistring desc = impulse_unistring_create_utf("channel to client lost", 8);
  impulse_unistring msg = NULL;
  
  impulse_channel_remote(client, &msg);
  
  impulse_error_raise(IMPULSE_ERROR_INFO, desc, msg);

//  impulse_hash_remove(impulse_channel_handle(channel));
  
  impulse_channel_close(client);
  impulse_channel_destroy(client);

  impulse_unistring_destroy(msg);
  impulse_unistring_destroy(desc);
}



impulse_bool client_event_handler(void *data)
{
  impulse_channel client = (impulse_channel)data;
  impulse_int pending = impulse_channel_pending(client);
  impulse_int got = 0;
  impulse_char buffer[1024];
  impulse_char *reply = "A reply from the Impulse test server\n";
  
  if (pending <= 0)
  {
    client_die(client);
    return 0;
  }
  
  /* Scale down bytes read to a manageable size */
  if (pending > 1023)
    pending = 1023;
  
  got = impulse_channel_receive(client, buffer, pending, NULL);
  if (got <= 0)
  {
    client_die(client);
    return 0;
  }
  
  buffer[got] = '\0';
  printf(buffer);
  impulse_channel_send(client, reply, strlen(reply), NULL);

  return 1;
}


impulse_bool channel_event_handler(void *data)
{
  impulse_channel channel = (impulse_channel)data;
  impulse_channel client;
  
  if (impulse_channel_accept(channel, &client) < 0)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot accept incoming channel", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, NULL);
  }
  else
  {
    impulse_unistring desc = impulse_unistring_create_utf("channel from client", 8);
    impulse_unistring msg = NULL;
    impulse_char *message = "A message from the Impulse test server\n";
    
    impulse_channel_send(client, message, strlen(message), NULL);
    
//    impulse_hash_store
    
    impulse_channel_handle(client, client_event_handler, client);

    impulse_channel_remote(client, &msg);
    impulse_error_raise(IMPULSE_ERROR_INFO, desc, msg);
    
    impulse_unistring_destroy(msg);
    impulse_unistring_destroy(desc);
  }
  return 1;
}


impulse_int main (impulse_int argc, 
		  impulse_char *argv[])
{
  impulse_channel channel = NULL;
  impulse_transport transport = NULL;
  impulse_unistring address;
  impulse_unistring help;
  impulse_command_value value;
  impulse_unistring arg;
  impulse_int caps = 0;
  
  impulse_init(argc, argv);
  
  address = impulse_unistring_create_utf("address", 8);
  help = impulse_unistring_create_utf("bind to local address [name]", 8);
  arg = impulse_unistring_create_utf("name", 8);
  impulse_command_option_arg(address, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  impulse_unistring_destroy(arg);
  
  impulse_command_parse();
  
  if (!impulse_command_set(address, &value))
    impulse_command_help();
  impulse_unistring_destroy(address);
  
  impulse_comms_init();
  
  if (!(transport = impulse_comms_lookup(value->s)))
  {
    impulse_unistring desc = impulse_unistring_create_utf("Unknown transport protocol requested", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value->s);
  }

  caps = IMPULSE_TRANSPORT_CLIENTSERVER | IMPULSE_TRANSPORT_CONNECTION | IMPULSE_TRANSPORT_RELIABLE |
    IMPULSE_TRANSPORT_SEQUENCED;
  
  if ((transport->caps & caps) != caps)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Transport is not capable of reliable client-server channels", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value->s);
  }
  
  channel = impulse_channel_create(transport);
  if (impulse_channel_open(channel, value->s, NULL, 20) == -1)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot open server channel", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value->s);
  }

  impulse_channel_handle(channel, channel_event_handler, channel);
  
  impulse_main();
  
  impulse_channel_close(channel);
  impulse_channel_destroy(channel);
  
  impulse_comms_cleanup();
  impulse_cleanup();
  
  return 0;
}
