/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * client.c
 * --------
 *
 * Communication channel functions
 * Client implementation
 */

#include "../../../config.h"

#include <impulse/comms/channel.h>
#include <impulse/util/memory.h>
#include <impulse/comms/transport_int.h>

struct _impulse_channel
{
  impulse_transport transport;
  void *data;
};


impulse_channel impulse_channel_create(impulse_transport transport)
{
  impulse_channel channel = NULL;
  void *data = NULL;
  
  if (!(data = transport->funcs->create()))
    return NULL;

  channel = impulse_alloc(sizeof(struct _impulse_channel));

  channel->transport = transport;
  channel->data = data;

  return channel;
}


void impulse_channel_destroy(impulse_channel channel)
{
  channel->transport->funcs->destroy(channel->data);
  impulse_free(channel);
}


/* Event functions */
impulse_int impulse_channel_handle(impulse_channel channel,
				   impulse_transport_handler func,
				   void *data)
{
  return channel->transport->funcs->handle(channel->data,
					   func,
					   data);
}


impulse_int impulse_channel_unhandle(impulse_channel channel)
{
  return channel->transport->funcs->unhandle(channel->data);
}


/* channel functions */
impulse_int impulse_channel_open(impulse_channel channel,
				 impulse_unistring local,
				 impulse_unistring remote,
				 impulse_int incoming)
{
  return channel->transport->funcs->open(channel->data,
					 local,
					 remote,
					 incoming);
}


impulse_int impulse_channel_accept(impulse_channel channel,
				   impulse_channel *incoming)
{
  void *data = NULL;
  impulse_int ret = 0;
 
  if ((ret = channel->transport->funcs->accept(channel->data,
					       &data)) < 0)
    return ret;
  
  *incoming = impulse_alloc(sizeof(struct _impulse_channel));
  (*incoming)->transport = channel->transport;
  (*incoming)->data = data;
  
  return 0;
}


impulse_int impulse_channel_close(impulse_channel channel)
{
  return channel->transport->funcs->close(channel->data);
}


impulse_int impulse_channel_active(impulse_channel channel)
{
  return channel->transport->funcs->active(channel->data);
}


/* Data transfer functions */
impulse_int impulse_channel_pending(impulse_channel channel)
{
  return channel->transport->funcs->pending(channel->data);
}


impulse_int impulse_channel_send(impulse_channel channel,
				 impulse_byte *buffer,
				 impulse_int size,
				 impulse_unistring address)
{
  return channel->transport->funcs->send(channel->data,
					 buffer,
					 size,
					 address);
}


impulse_int impulse_channel_receive(impulse_channel channel,
				    impulse_byte *buffer,
				    impulse_int size,
				    impulse_unistring *address)
{
  return channel->transport->funcs->receive(channel->data,
					    buffer,
					    size,
					    address);
}


impulse_int impulse_channel_peek(impulse_channel channel,
				 impulse_byte *buffer,
				 impulse_int size,
				 impulse_unistring *address)
{
  return channel->transport->funcs->peek(channel->data,
					 buffer,
					 size,
					 address);
}


/* Address functions */
impulse_int impulse_channel_local(impulse_channel channel,
				  impulse_unistring *address)
{
  return channel->transport->funcs->local(channel->data,
					  address);
}


impulse_int impulse_channel_remote(impulse_channel channel,
				   impulse_unistring *address)
{
  return channel->transport->funcs->remote(channel->data,
					   address);
}


/* End of file */
