/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * box.h
 * -----
 *
 * Widgets with a bounding box
 */

#ifndef __impulse_rows_box_h
#define __impulse_rows_box_h

#include <impulse/rows/widget.h>
#include <impulse/util/measure.h>

/*                            ======--------======                           */

/* Box widget type */

typedef void **ROWS_box;

/* Other rectangular types */

struct _ROWS_point {
  microns x,y;
};

typedef struct _ROWS_point ROWS_point;

struct _ROWS_rect {
  ROWS_point pos;
  ROWS_point size;
};

typedef struct _ROWS_rect ROWS_rect;

struct _ROWS_box_info {
  ROWS_point position;
  ROWS_point virtual;
  ROWS_point physical;

  ROWS_point scroll;
};

typedef struct _ROWS_box_info ROWS_box_info;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget     ROWS_box_create         (void);
extern ROWS_widget    *ROWS_box_initialise     (int id);
extern inline int      ROWS_box_next_identifier(void);
extern inline ROWS_box ROWS_box_cast           (void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

extern void               ROWS_box_set_position  (ROWS_box    box,
						  ROWS_point *position);
extern inline ROWS_point *ROWS_box_get_position  (ROWS_box    box);
extern void               ROWS_box_set_boundaries(ROWS_box    box,
						  ROWS_point *boundaries);
extern impulse_inline void ROWS_box_get_boundaries(ROWS_box   box,
						   ROWS_rect *rect);

extern void               ROWS_box_set_physical (ROWS_box    box,
						 ROWS_point *physical);
extern void               ROWS_box_set_virtual  (ROWS_box    box,
						 ROWS_point *virtual);
extern impulse_inline ROWS_point *ROWS_box_get_physical (ROWS_box    box);
extern impulse_inline ROWS_point *ROWS_box_get_virtual  (ROWS_box    box);

extern void               ROWS_box_set_scroll   (ROWS_box    box,
						 ROWS_point *scroll);
extern impulse_inline ROWS_point *ROWS_box_get_scroll   (ROWS_box    box);

extern impulse_inline void        ROWS_box_get_info     (ROWS_box       box,
							 ROWS_box_info *info);

extern impulse_bool ROWS_box_contains(ROWS_box box,
				      ROWS_point *point);

#endif
