/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Charles Talibard (for the idea)
 */

/*
 * constructor.h
 * -------------
 *
 * A widget used for constructing complex user interface artifacts :)
 * (Or, put more simply, Charles's guideline widget idea)
 */

#ifndef __impulse_rows_constructor_h
#define __impulse_rows_constructor_h

#include <impulse/rows/container.h>

/*                            ======--------======                           */

/* Box widget type */

typedef void **ROWS_constructor;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget             ROWS_constructor_create         (void);
extern ROWS_widget            *ROWS_constructor_initialise     (int id);
extern inline int              ROWS_constructor_next_identifier(void);
extern inline ROWS_constructor ROWS_constructor_cast           (void **widget);

/*                            ======--------======                           */

/* Pre-realisation functions */

extern void ROWS_constructor_set_ideal_size(ROWS_constructor  widget,
					    ROWS_rect        *size);

/*                            ======--------======                           */

/* Guideline creation and manipulation functions */

typedef enum {
  ROWS_CONSTRUCTOR_VERTICAL,
  ROWS_CONSTRUCTOR_HORIZONTAL
  /* ROWS_CONSTRUCTOR_DIAGONAL */ /* Future expansion :-) */
} ROWS_constructor_orientation;

typedef enum {
  ROWS_CONSTRUCTOR_MIN,      /* At minimum bound */
  ROWS_CONSTRUCTOR_MAX,      /* At maximum bound */
  ROWS_CONSTRUCTOR_RELATIVE, /* Relative to two other guidelines */
  ROWS_CONSTRUCTOR_ABSOLUTE  /* Absolute offset from one other guideline */
} ROWS_constructor_type;

#define ROWS_CONSTRUCTOR_GL_LEFT 0
#define ROWS_CONSTRUCTOR_GL_RIGHT 1
#define ROWS_CONSTRUCTOR_GL_TOP 2
#define ROWS_CONSTRUCTOR_GL_BOTTOM 3

extern int  ROWS_constructor_add_GL     (ROWS_constructor             widget,
					 ROWS_constructor_orientation orient,
					 ROWS_constructor_type        type,
					 int   lower_guideline,
					 int   upper_guideline,
					 float position);
extern void ROWS_constructor_GL_destroy (ROWS_constructor widget,
					 int              guideline);

/* 
 * Note that these functions are not thread-safe. Call ROWS_widget_lock 
 * before calling them.
 */
extern int  ROWS_constructor_GL_position(ROWS_constructor widget,
					 int              guideline);
extern ROWS_constructor_orientation ROWS_constructor_GL_orient(ROWS_constructor widget,
							       int  guideline);

/* Legibility functions */
#define ROWS_constructor_add_guideline      ROWS_constructor_add_GL
#define ROWS_constructor_guideline_destroy  ROWS_constructor_GL_destroy
#define ROWS_constructor_guideline_position ROWS_constructor_GL_position

/*                            ======--------======                           */

/* Routines to attach widgets to guidelines */

extern void ROWS_constructor_add_child(ROWS_constructor cons,
				       ROWS_child       child,
				       int              GL_left,
				       int              GL_right,
				       int              GL_top,
				       int              GL_bottom);
extern void ROWS_constructor_remove_child(ROWS_constructor cons,
					  ROWS_child child);

#endif
