dnl 
dnl Macros to determine what signal support is available
dnl


dnl *****************************************************************
dnl * Initialise configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_INIT,
[
  SIGNAL_SIGINFO="0"
  SIGINFO_BUG="0"
])


dnl *****************************************************************
dnl * Check signal handling capabilities
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_CHECK,
[
  IMPULSE_MSG_SECTION([Checking signal handling])
  
  IMPULSE_SIGNAL_CHECK_SIGINFO
  if test "$USE_THREADS" = "1" -a "$THREAD_LIBRARY" = "POSIX" ; then
    IMPULSE_SIGNAL_CHECK_THREAD
  fi
])


dnl *****************************************************************
dnl * Check to see if siginfo is available
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_CHECK_SIGINFO,
[
  IMPULSE_PARAMS_SAVE
  
  AC_CHECK_FUNC(sigqueue,,
    AC_CHECK_LIB(rt, sigqueue,SIGNAL_LIBS="-lrt",
      AC_CHECK_LIB(posix4, sigqueue,SIGNAL_LIBS="-lposix4")))
  
  LIBS="$LIBS $SIGNAL_LIBS"
  
  AC_CACHE_CHECK([for POSIX 1003.1b extra signal information], impulse_cv_siginfo,
  [
    AC_TRY_RUN([
      #include <sys/types.h>
      #include <signal.h>
      #include <unistd.h>
      #include <stdio.h>
      
      void bing(int sig, siginfo_t *info, void *data)
      {
        if (info == NULL)
  	  exit(1);
	  
	if (info->si_value.sival_int == 22)
	  exit(0);

        exit(1);
      }
      
      #define SIGNAL SIGRTMIN+1
      
      int main()
      {
        struct sigaction act, oact;
        int result;
        union sigval value;
      
        sigfillset(&act.sa_mask);
        act.sa_flags     = SA_SIGINFO;
	act.sa_sigaction = bing;
	
	result = sigaction(SIGNAL, &act, &oact);
	
	if (result != 0)
          exit(1);
      
        value.sival_int = 22;
	result = sigqueue(getpid(), SIGNAL, value);
	
	if (result != 0)
	  exit(1);
      
        sleep(10);
	
	exit(1);
      }
    ],[
      impulse_cv_siginfo="yes"
    ],[
      impulse_cv_siginfo="no"
    ],[
      impulse_cv_siginfo="no (guessed for x-compile)"
    ])
  ])

  if test "$impulse_cv_siginfo" = "yes" ; then
    SIGNAL_SIGINFO="1"
  else
    SIGNAL_SIGINFO="0"
  fi

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check for siginfo linuxthread bug
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_CHECK_THREAD,
[
  AC_CACHE_CHECK([for linux threads signal bug], impulse_cv_signalbug,
  [
    AC_TRY_RUN([
      #include <sys/types.h>
      #include <signal.h>
      #include <unistd.h>
      #include <stdio.h>
      
      #define SIGNAL SIGRTMIN+1
      
      void bing(int sig, siginfo_t *info, void *data)
      {
        if (info == NULL)
	  exit(1);
	  
	if (info->si_value.sival_int == 565)
          exit(0);

        exit(0);
      }

      int main()
      {
        struct sigaction act, oact;
	int result;
	union sigval value;
	
	sigfillset(&act.sa_mask);
	act.sa_flags     = SA_SIGINFO;
	act.sa_sigaction = bing;
	
	result = sigaction(SIGNAL, &act, &oact);
	
	if (result != 0)
  	  exit(1);
	
	value.sival_int = 565;
	result = sigqueue(getpid(), SIGNAL, value);
	
	if (result != 0)
	  exit(1);
	
	sleep(10);
	exit(1);
      }
    ],[
      impulse_cv_signalbug="no"
    ],[
      impulse_cv_signalbug="yes"
    ],[
      impulse_cv_signalbug="yes (guessing for x-compile)"
    ])
  ])
  
  if test "$impulse_cv_signalbug" = "no" ; then
    SIGINFO_BUG=0
  else
    SIGINFO_BUG=1
    SIGNAL_SIGINFO=0
  fi
])


dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_FINISH,
[
  if test "$SIGNAL_SIGINFO" = "1" ; then
    AC_DEFINE(HAVE_SIGINFO, 1)
  fi
  AC_SUBST(SIGNAL_CFLAGS)
  AC_SUBST(SIGNAL_LDFLAGS)
  AC_SUBST(SIGNAL_LIBS)
])

dnl *****************************************************************
dnl * Print a configuration summary
dnl *****************************************************************

AC_DEFUN(IMPULSE_SIGNAL_SUMMARY,
[
  if test "$SIGINFO_BUG" = "1" ; then
    IMPULSE_ECHO([Your linux threads library has a fatal siginfo bug])
    IMPULSE_ECHO([Disabling siginfo support])
    IMPULSE_ECHO([Please upgrade to a newer version])
  fi
])

dnl End of file
